/* libsizzle/modules.h - module handling functions
 
  Copyright (C) 2000 Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
 
  This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This software is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this package; see the file COPYING.  If not, write to the
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.  */

#ifndef __SIZZLE_MODULE_H__
#define __SIZZLE_MODULE_H__

#if HAVE_CONFIG_H
# include <config.h>
#endif 

#if __SIZZLE_IMPORT__
# define SIZZLE_API __declspec (dllimport)
#elif __SIZZLE_EXPORT__ 
# define SIZZLE_API __declspec (dllexport)
#else
# define SIZZLE_API
#endif

SIZZLE_API extern zzz_object_t zzz_tagged_type_module;

#define module_p(c)       (tagged_p(c) && \
                           tagged_type(c) == zzz_tagged_type_module)
#define module_desc(c)    ((module_desc_t) (tagged_data0(c)))
#define module_name(c)    (module_desc(c)->name)
#define module_exports(c) (module_desc(c)->exports)
#define module_private(c) (module_desc(c)->private)
#define module_loaded(c)  (module_desc(c)->loaded)

typedef struct module_descriptor * module_desc_t;
struct module_descriptor
{
  zzz_scm_t name;		/* List, the module specification. */
  zzz_scm_t exports;		/* List containing exported symbols. */
  zzz_scm_t private;		/* Private environment. */
  int loaded;			/* Was the module loaded from a file?  */
};

SIZZLE_API zzz_scm_t zzz_make_module (zzz_scm_t name);

SIZZLE_API result_t zzz_init_modules (void);

SIZZLE_API result_t zzz_c_create_module (char * cname, zzz_scm_t * ret_module);

SIZZLE_API result_t zzz_c_module_define (zzz_scm_t module, char * cname,
					 zzz_scm_t value, int export,
					 zzz_scm_t * result);

#endif /* not __SIZZLE_MODULE_H__ */
