/* libsizzle/uvector64.h - 64-bit uniform vector function declarations
 
  Copyright (C) 2000 Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
 
  This is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This software is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this package; see the file COPYING.  If not, write to the
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.  */

#ifndef __SIZZLE_UVECTOR64_H__
#define __SIZZLE_UVECTOR64_H__

#if HAVE_CONFIG_H
# include <config.h>
#endif 

#if __SIZZLE_IMPORT__
# define SIZZLE_API __declspec (dllimport)
#elif __SIZZLE_EXPORT__ 
# define SIZZLE_API __declspec (dllexport)
#else
# define SIZZLE_API
#endif

#include "node.h"

#if HAVE_LONG_LONG

#define s64vector_p(c)     (tagged_p(c) && \
                           tagged_type(c) == zzz_tagged_type_s64vector)
#define ro_s64vector_p(c)  (s64vector_p(c) && \
                           tagged_info(c) == tagged_info_ro)
#define s64vector_val(c)   (((signed __INT64__ *) tagged_data0(c)))
#define s64vector_len(c)   ((zzz_object_t) tagged_data1(c))

#define u64vector_p(c)     (tagged_p(c) && \
                           tagged_type(c) == zzz_tagged_type_u64vector)
#define ro_u64vector_p(c)  (u64vector_p(c) && \
                           tagged_info(c) == tagged_info_ro)
#define u64vector_val(c)   (((unsigned __INT64__ *) tagged_data0(c)))
#define u64vector_len(c)   ((zzz_object_t) tagged_data1(c))

SIZZLE_API extern zzz_object_t zzz_tagged_type_s64vector;
SIZZLE_API extern zzz_object_t zzz_tagged_type_u64vector;

/* Create a vector object holding @var{len} objects of the type
   indicated by the tag.  */
SIZZLE_API zzz_scm_t zzz_make_s64vector (int len);
SIZZLE_API zzz_scm_t zzz_make_u64vector (int len);

/* Create a constant vector object holding @var{len} objects of the
   type indicated by the tag.  Vectors created using this functions
   cannot be modified using Scheme primitives like
   @code{TAGvector-set!}.  */
SIZZLE_API zzz_scm_t zzz_make_ro_s64vector (int len);
SIZZLE_API zzz_scm_t zzz_make_ro_u64vector (int len);

#endif /* HAVE_LONG_LONG */

/* Init function for the 64-bit uniform vector module.  Defines the
   64-bit uniform vector types and all Scheme primitives for dealing
   with these uniform vector objects.  */
SIZZLE_API result_t zzz_init_uvector64 (void);

#endif /* not __SIZZLE_UVECTOR64_H__ */
